// ----------------------------------------------------------------------------
// ale_config.cxx
//
// Copyright (C) 2020
//		Dave Freese, W1HKJ
//
// This file is part of fldigi.
//
// Fldigi is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Fldigi is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------

#include <config.h>

#include <iostream>
#include <fstream>
#include <string>

#include <FL/Fl_Preferences.H>

#include "ale_dialog.h"
#include "ale_config.h"
#include "main.h"
#include "fl_digi.h"

ale_struct ale_config = {
	50,					// int X;
	50,					// int Y;
	680, 				// int W;
	400, 				// int H;

	true,				// bool rx_topline;
	true,				// bool	anycall;
	false,				// bool	allcall;
	false,				// bool sounding;

	"",					// std::string	address;
	"",					// std::string	selcall_address4;
	"",					// std::string	selcall_address6;
	"1843:0,3596:1,7102:1,10145.5:0,14109:1,18106:0,21096:0,24926:0,28146:0",	// std::string	scn_frequencies;
	"1843:0,3596:1,7102:1,10145.5:0,14109:1,18106:0,21096:0,24926:0,28146:0",	// std::string	snd_frequencies;

/*
  nets:
    one line for each network, each line delineated by brackets {}
    elements of each network delineated by comma
      {netname , netaddr , myaddr , active , channel # , channel # , ...}{...}...
    where:
      netname  - noun name of network
      netaddr  - net address, from 1 to 3 characters
      myaddr   - my address, from 1 to 15 characters
      active   - activate this network
      channels - separated by commas
*/
"\
{HF all,n/a,W1HKJ,ch2,ch3,ch4}\
{VHF local,HAC,W1HKJ,ch10,ch11}\
",
/*
  channels:
    one line for each channel, each line delineated by brackets {}
    channel element, each delineated by comma
      chd, rrrrrr.r , scn , tttttt.t , snd , mode , snd_int
   where:
      chd      - channel designator, noune name
      rrrrrr.r - channel receive frequency
      rmode    - receive mode (L-lsb, U-usb, A-am, F-fm)
      scn      - boolean T/F scan enabled
      tttttt.t - channel transmit frequency
      tmode    - transmit mode (L-lsb, U-usb, A-am, F-fm)
      snd      - boolean T/F sound enabled
      snd_int  - sounding interval in minutes (nominal 30)
*/
"\
{ch1, 1843.0  , U , 0 ,  1833.0 , U , 0 , 30}\
{ch2, 3596.0  , U , 1 ,  3596.0 , U , 0 , 30}\
{ch3, 7102.0  , U , 1 ,  7102.0 , U , 1 , 30}\
{ch4, 10145.5 , U , 0 , 10145.5 , U , 0 , 30}\
{ch5, 14109.0 , U , 0 , 14109.0 , U , 1 , 30}\
{ch6, 18106.0 , U , 0 , 18106.0 , U , 0 , 30}\
{ch7, 21096.0 , U , 0 , 21096.0 , U , 0 , 30}\
{ch8, 24926.0 , U , 0 , 24926.0 , U , 0 , 30}\
{ch9, 28146.0 , U , 0 , 28146.0 , U , 0 , 30}\
{ch10,144185.0, U , 1 ,144185.0 , U , 1 , 30}\
{ch11,146610.1, F , 0 ,146610.0 , F , 1 , 120}\
",

	60,					// int		aging;

	30,					// double	link_timeout;
	6,					// double	call_timeout;
	30,					// double	ack_timeout;
	3,					// double	listen;
	300,				// double	dwell_time;
	6,					// double	sounding_interval;
	2,					// double	call_retries;
	5,					// double	repeats;
	30,					// double	snd_hangup;
	1,					// int		scan_rate;
	1,					// int		decode_errors
	100,				// double	tune;

//----------------------------------------------------------------------
	false				// bool ale_config_last;
};

void ale_struct::save_ale_config()
{
	Fl_Preferences ale_pref(ALE_dir.c_str(), "w1hkj.com", "fldigi_ale");

	ale_pref.set("version", PACKAGE_VERSION);

	int mX = ale_dialog->x();
	int mY = ale_dialog->y();
	if (mX >= 0 && mX >= 0) {
		X = mX;
		Y = mY;
	}
	W = ale_dialog->w();
	H = ale_dialog->h();

// bools
	ale_pref.set("rx_topline", rx_topline);
	ale_pref.set("anycall", anycall);
	ale_pref.set("allcall", allcall);
	ale_pref.set("sounding", sounding);

// ints
	ale_pref.set("X", X);
	ale_pref.set("Y", Y);
	ale_pref.set("W", W);
	ale_pref.set("H", H);

	ale_pref.set("aging", aging);

// doubles
	ale_pref.set("link_timeout", link_timeout);
	ale_pref.set("call_timeout", call_timeout);
	ale_pref.set("ack_timeout", ack_timeout);
	ale_pref.set("listen", listen);
	ale_pref.set("dwell_time", dwell_time);
	ale_pref.set("sounding_interval", sounding_interval);
	ale_pref.set("call_retries", call_retries);
	ale_pref.set("repeats", repeats);
	ale_pref.set("snd_hangup", snd_hangup);
	ale_pref.set("scan_rate", scan_rate);
	ale_pref.set("decode_errors", decode_errors);
	ale_pref.set("tune", tune);

// strings
	ale_pref.set("address", address.c_str());
	ale_pref.set("selcal_address4", selcall_address4.c_str());
	ale_pref.set("selcal_address6", selcall_address6.c_str());

	ale_pref.set("scn_frequencies", scn_frequencies.c_str());
	ale_pref.set("snd_frequencies", snd_frequencies.c_str());
	ale_pref.set("nets", nets.c_str());
	ale_pref.set("channels", channels.c_str());

}

void ale_struct::load_ale_config()
{
	Fl_Preferences ale_pref(ALE_dir.c_str(), "w1hkj.com", "fldigi_ale");

	int i;
	char strbuff[1000];
	char version[64]; version[sizeof(version)-1] = '\0';

	// Skip loading the rest of the ale_config variables if we didn't read a
	// version name/value pair; or this is not a file that supports dual
	// channel browsers.
	ale_config_last = ale_pref.get("version", version, "", sizeof(version)-1);
	if (!ale_config_last)
		return;

// bools
	ale_pref.get("rx_topline", i, rx_topline); rx_topline = i;
	ale_pref.get("anycall", i, anycall); anycall = i;
	ale_pref.get("allcall", i, allcall); allcall = i;
	ale_pref.get("sounding", i, sounding); sounding = i;

// ints
	ale_pref.get("X", X, X);
	ale_pref.get("Y", Y, Y);
	ale_pref.get("W", W, W);
	ale_pref.get("H", H, H);

	ale_pref.get("aging", aging, aging);

// doubles

	ale_pref.get("link_timeout", link_timeout, link_timeout);
	ale_pref.get("call_timeout", call_timeout, call_timeout);
	ale_pref.get("ack_timeout", ack_timeout, ack_timeout);
	ale_pref.get("listen", listen, listen);
	ale_pref.get("dwell_time", dwell_time, dwell_time);
	ale_pref.get("sounding_interval", sounding_interval, sounding_interval);
	ale_pref.get("call_retries", call_retries, call_retries);
	ale_pref.get("repeats", repeats, repeats);
	ale_pref.get("snd_hangup", snd_hangup, snd_hangup);
	ale_pref.get("scan_rate", scan_rate, scan_rate);
	ale_pref.get("decode_errors", decode_errors, decode_errors);
	ale_pref.get("tune", tune, tune);

// strings
	memset(strbuff, 0, 1000);
	ale_pref.get("address", strbuff, "", sizeof(strbuff) - 1);
	address = strbuff;

	memset(strbuff, 0, 1000);
	ale_pref.get("selcal_address4", strbuff, "", sizeof(strbuff) - 1);
	selcall_address4 = strbuff;

	memset(strbuff, 0, 1000);
	ale_pref.get("selcal_address6", strbuff, "", sizeof(strbuff) - 1);
	selcall_address6 = strbuff;

	memset(strbuff, 0, 1000);
	ale_pref.get("scn_frequencies", strbuff, scn_frequencies.c_str(), sizeof(strbuff) - 1);
	scn_frequencies = strbuff;

	memset(strbuff, 0, 1000);
	ale_pref.get("snd_frequencies", strbuff, snd_frequencies.c_str(), sizeof(strbuff) - 1);
	snd_frequencies = strbuff;

	memset(strbuff, 0, 1000);
	ale_pref.get("nets", strbuff, nets.c_str(), sizeof(strbuff) - 1);
	nets = strbuff;

	memset(strbuff, 0, 1000);
	ale_pref.get("channels", strbuff, channels.c_str(), sizeof(strbuff) - 1);
	channels = strbuff;

}

