// =====================================================================
// 
// Copyright (C) 2000 - 2020 
//   Charles Brain <chbrain@dircon.co.uk>
//   Ilkka Toivanen <pile@aimo.kareltek.fi>
//   David Freese <w1hkj@bellsouth.net>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// File: ale_codec.h
//
// =====================================================================

/*
  Modem signal processing routine
*/
#ifndef ALE_CODEC_H
#define ALE_CODEC_H

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <stdio.h>

#include <string>

#include "modem.h"
#include "filters.h"

#include "ale_golay.h"

#define ALE_FFT_LEN            64
#define MOD_64                 64
#define SYMBOLS_PER_WORD       49
#define VOTE_BUFFER_LENGTH     48
#define BITS_PER_SYMBOL         3
#define VOTE_ARRAY_LENGTH      SYMBOLS_PER_WORD * BITS_PER_SYMBOL
#define BAD_VOTE_THRESHOLD     25
#define SIG_FILT              128
#define NOISE_FILT           1024

#define ASCII_128  0
#define ASCII_64   1
#define ASCII_38   2

#define ALE_SAMPLERATE 8000

using namespace std;

class ale_codec : public modem {

	struct ale_complex {
		double real;
		double imag;
	};

public:
	enum { DAT, THRU, TO, TWS, FROM, TIS, CMD, REP };
	enum { START, MID, END };
  
private:
	ale_golay  *golay;

	static std::string	preamble_strs[];
	static std::string	command_strs[];

	static char		ASCII_Set[];
	static const	int symbol_lookup[];
	static const	int vote_lookup[];

	static double	outbuf[];
	static int		oct_freq[];

// FFT information
	double			fft_cs_twiddle[ALE_FFT_LEN];
	double			fft_ss_twiddle[ALE_FFT_LEN];
	double			fft_history[ALE_FFT_LEN];
	double			fft_mag[ALE_FFT_LEN];
	int				fft_history_offset;
	ale_complex		fft_out[ALE_FFT_LEN];
	double			s2n;
	double			noise_signal;
	double			ale_signal;
	double			ratio;
	double			sinad;

	int		max_offset;
	double	temp_real;
	double	temp_imag;
	double	max_magnitude;
	double	min_magnitude;

	int	last_symbol;
	int	last_sync_position;
	int	sample_count;

	Cmovavg sig_filter[8];
	Cmovavg noise_filter;

// sync information
	double			mag_sum[ALE_FFT_LEN];
	double			mag_history[ALE_FFT_LEN][SYMBOLS_PER_WORD];
	int				mag_history_offset;
	int				bit_count;
	bool			word_sync;

	int				bits[VOTE_ARRAY_LENGTH];
	int				input_buffer_pos;
	int				word_sync_position;
	int				majority_vote_array[VOTE_BUFFER_LENGTH];
	int				ber, sum, errors;
	unsigned long	word;

	unsigned long	worda;
	unsigned long	wordb;
	int				error_a;
	int				error_b;
	int				preamble;
	int				command;
	string			address;

	time_t			timestamp;
	struct			tm *UTC_time;

	std::string		tx_string;
	std::string		tx_preamble_string;
	int				tx_preamble;

//	void			decodeCMD (unsigned long word);
	std::string			decode_word (unsigned long word);
	unsigned long	modem_de_interleave_and_fec(int *input, int *errors);
	void			modem_new_symbol(int sym);

	unsigned long	encodeCMD(int cmd);
	unsigned long	encode_word(long words);

public:
	ale_codec();
	~ale_codec();

	void init(void);

	void tx_init() {}
	void rx_init() {}
	void restart() {}

//	void rx_process(unsigned short *sample, int length);
	int  rx_process(const double *sample, int length);

	int  tx_process();

	std::string tx_fec(unsigned long ALE_WORD);
	void xmt_ale_octet(int octet, int which);

	void xmt_ale_triplet(std::string pre, std::string data);

	std::string octet_str(std::string binary);
	std::string binary_str(unsigned long nbr, int size);

//	void ModulateXmtr(double *outbuf, int n);
};

#endif

